package FLYLib;

use Math::CDF;
use Scalar::Util;
use Carp;
use strict;
require Exporter;

our @ISA = qw(Exporter);
our @EXPORT = qw(sign sum mean geometric_mean sd var median quantile iqr sum order sort_array restore inverse t
                 max min cov cor apply mapply happly tapply choose frequency normalize scale sample repeat unique
                 subset which intersect union complement rnorm bipnorm paste seq c print_ref
                 is_numeric is_array_ref is_hash_ref is_scalar_ref is_code_ref is_ref_ref);

1;

#===== statistic functions ==================

sub sign {
    if($_[0] > 0) {
        return 1;
    } elsif($_[0] < 0) {
        return -1;
    } else {
        return 0;
    }
}

sub sum {
    my $array = shift;
    
    my $sum = 0;
    for(my $i = 0; $i < scalar(@$array); $i ++) {
        $sum += $array->[$i];
    }
    return $sum;
}

sub mean {
    return sum($_[0]) / scalar(@{$_[0]});
}

sub geometric_mean {
    return exp(sum(apply($_[0], "log")) / scalar(@{$_[0]}));
}

sub sd {
    return sqrt(var(@_));
}

sub var {
    my $array = shift;
    my $mean = shift;
    
    if($mean eq "") {
        $mean = mean($array);
    }
    return sum(apply($array, sub {($_[0]-$mean)**2})) / $#$array;
}

sub cov{
    my $array1 = shift;
    my $array2 = shift;
	
	if(scalar(@$array1) != scalar(@$array2)) {
		croak "ERROR: Length of the two vectors should be same";
	}
	
    my $mean1 = mean($array1);
    my $mean2 = mean($array2);

    my $sum = 0;
    for(my $i = 0; $i < scalar(@$array1); $i ++) {
        $sum += ($array1->[$i] - $mean1)*($array2->[$i] - $mean2);
    }
    return $sum / $#$array1;
}

sub cor {
    my $array1 = shift;
    my $array2 = shift;
    
    return cov($array1, $array2)/sd($array1)/sd($array2);
}

#======= descriptive functions =============================
sub max {
    my $array = shift;
    my $max = $array->[0];
    foreach (@$array) {
        $max = $max > $_ ? $max : $_;
    }
    return $max;
}

sub min {
    my $array = shift;
    my $min = $array->[0];
    foreach (@$array) {
        $min = $min < $_ ? $min : $_;
    }
    return $min;
}

sub median {
    my $array = shift;
    
    my $sort = sort_array($array);
    
    my $median_index;
    if(scalar(@$sort) % 2 == 1) {
        $median_index = int(scalar(@$sort) / 2);
        return $sort->[$median_index];
    }
    else {
        return ($sort->[int(scalar(@$sort) / 2)] + $sort->[int(scalar(@$sort) / 2) - 1])/2;
    }
}
sub quantile {
    my $array = shift;
    my $q = shift;
    my $sort = sort_array($array);
    
    my $median_index;
	if ($q>scalar(@$sort)) {
		$q=scalar(@$sort);
	}
	if ($q<1) {
		$median_index=int((scalar(@$sort)-1)*$q);
	}else{
		$median_index=$q-1;
	}
	return $sort->[$median_index];	
}


# if quantile is a number, then a quantile returns
# if quantile is a array reference, then a quantile array reference returns
# λ01֮Ե
#sub quantile {
#    my $array = shift;
#    my $q = shift;
#    $q = $q ne "" ? $q : [0, 0.25, 0.5, 0.75, 1];
#    
#    my $max = max($array);
#	my $min = min($array);
#    
#    if(is_array_ref($q)) {
#        return apply($q, sub{ $_[0]*($max - $min) + $min });
#    }
#    else {
#        return $q*($max - $min) +$min;
#    }
#}

sub iqr {
	my $array = shift;
	
	my $q = quantile($array, [.25, .75]);
	return $q->[1] - $q->[0];
}

#======= array operation ================================
sub order {
    my $array = shift;
    my $desc = shift;   # ڶʾӴС
    
    my $hash;
    for (0..$#$array) {
        $hash->{$_} = $array->[$_];
    }
    
    my $order = [];
    if($desc) {
        foreach (sort { $hash->{$b} <=> $hash->{$a} } keys %$hash) {
            push(@$order, $_);
        }
    }
    else {
        foreach (sort { $hash->{$a} <=> $hash->{$b} } keys %$hash) {
            push(@$order, $_);
        }
    }
    return $order;
}

# С
sub sort_array {
	my $array = shift;
    my $desc = shift;   # ڶʾӴС
    
	return [@$array[@{order($array, $desc)}]];
}

# Ź鰴orderԭ
sub restore {
	my $sort = shift;
	my $order = shift;
	
	my $restore = [];
	for (0..$#$order) {
		$restore->[$order->[$_]] = $sort->[$_];
	}
	return $restore;
}

# ת
sub inverse {
    my $array = shift;
    
    return [@$array[@{seq($#$array, 0)}]];
}

# ת
sub t {
    my $matrix = shift;
    
    my $n_row = scalar(@$matrix);
    my $n_col = scalar(@{$matrix->[0]});
    
    my $t = [[]];
    for(my $i = 0; $i < $n_row; $i ++) {
        for(my $j = 0; $j < $n_col; $j ++) {
            $t->[$j]->[$i] = $matrix->[$i]->[$j];
        }
    }
    return $t;
}

# apply a function on every element of an array
sub apply {
    my $array = shift;
    my $function = shift;

    my $apply = [];
    if(is_code_ref($function)) {
        @$apply = map { &$function($_) } @$array;
    }
    else {
        @$apply = map { eval("$function(\$_)") } @$array;
        croak $@ if $@;
    }
    return $apply;
}

# ҪеapplyжʹøĽmapply
# Ӧÿ mapply([1..10], [1..10], sub{mean(\@_)})mapply([1..10], 1, sub{$_[0]-$_[1]})ļ
sub mapply {
    my $function = pop; # the last argument
    my $is_function = is_code_ref($function);
    my @array = @_;

    for (0..$#array) {
        if(! is_array_ref($array[$_])) {
            $array[$_] = [$array[$_]];
        }
    }

    my $length = apply(\@array, sub{scalar(@{$_[0]})});
    my $max_length = max($length);
	
    @array = @{ apply(\@array, sub{_resize($_[0], $max_length)}) };
	
    my $mapply = [];
    for my $i (0..($max_length-1)) {
        my $param = apply(\@array, sub {$_[0]->[$i]});
        if($is_function) {
            $mapply->[$i] = &$function(@$param);
        }
        else {
            $mapply->[$i] = eval("$function(\@\$param)");
            croak $@ if $@;
        }
    }

    return $mapply;
}

# һsizeĴСнضϻѭ
sub _resize {
    my $array = shift;
    my $size = shift || scalar(@$array);
    my $scalar = scalar(@$array);

    if($size == $scalar) {
        return $array;
    }
    elsif($size < $scalar) {
        $size --;
        return subset($array, [0..$size]);
    }
    else {
        $size --;
        my $index = apply([0..$size], sub {$_ % $scalar});
        return subset($array, $index);
    }
}

# hashÿһvalueв
sub happly {
    my $hash = shift;
    my $function = shift;

    my $happly = [];
    foreach (keys %$hash) {
        if(is_code_ref($function)) {
            $happly->{$_} = &$function($hash->{$_})
        }
        else {
            $happly->{$_} = eval("$function(\$hash->{\$_})");
            croak $@ if $@;
        }
    }
    return $happly;
}

sub tapply {
    my $array = shift;
	my $function = pop;
    my @category = @_;
	
	my $length = apply(\@category, sub{scalar(@{$_[0]})});
	push(@$length, scalar(@$array));
	if(max($length) != min($length)) {
		croak "ERROR: Length of the vector must be equal to the length of all categories.\n";
	}
	
	my $category = paste(@category, "|");

    my $label = unique($category);
    my $tapply = {};
    for (0..$#$label) {
        my $current_label = $label->[$_];
        my $index = choose($category, sub {$_[0] eq $current_label});
        $index = which($index);
		my @data = @{subset($array, $index)};
        if(is_code_ref($function)) {
            $tapply->{$current_label} = &$function(@data);
        }
        else {
            $tapply->{$current_label} = eval("$function(\@data)");
            croak $@ if $@;
        }
    }
    return $tapply;
}

sub choose {
    my $array = shift;
    my $function = shift;

    my $choose = [];
    for (0..$#$array) {
        if(is_code_ref($function)) {
            $choose->[$_] = $function->($array->[$_]) ? 1 : 0;
        }
        else {
            $choose->[$_] = eval("$function(\$array->[$_]) ? 1 : 0");
            croak $@ if $@;
        }
    }
    return $choose;
}


# ظһֵɴ
sub repeat {
    my $value = shift;
    my $size = shift;
    
    my $array = [];
    for(my $i = 0; $i < $size; $i ++) {
        push(@$array, $value);
    }
    return $array;
}

# ÿֵƵ,Ƕ
sub frequency {
    my @category = @_;
    
    my $f = {};
	my $category = paste(@category, "|");
    foreach (@$category) {
		$f->{$_} ++;
	}
    return $f;
}

# һǷȥֵԼǷ
sub normalize {
    my $array = shift;
    my $scale_outlier_elimination = shift;
    my $class_label = shift;
    
    $class_label = is_array_ref($class_label) ? $class_label : repeat(0, scalar(@$array));
    
    # Ҫȥֵ
    if($scale_outlier_elimination) {
        # ÿһֵһhasharray
        my $array_class;
        for(my $i = 0; $i < scalar(@$class_label); $i ++) {
            push(@{$array_class->{$class_label->[$i]}}, $array->[$i]);
        }
        
        # 25λ
        my $q;
        my $iqr;
        foreach (keys %$array_class) {
            $q->{$_} = quantile($array_class->{$_}, [0.25, 0.75]);
            $iqr->{$_} = $q->{$_}->[1] - $q->{$_}->[0];
        }
        
        # һɾֵ֮array
        my $array_filtered;
        for(my $i = 0; $i < scalar(@$class_label); $i ++) {
            if($array->[$i] > $q->{$class_label->[$i]}->[0] - 1.5*$iqr->{$class_label->[$i]}
               and $array->[$i] < $q->{$class_label->[$i]}->[1] + 1.5*$iqr->{$class_label->[$i]}){
                push(@$array_filtered, $array->[$i]);
            }
        }
        
        my $mean = mean($array_filtered);
        my $sd = sd($array_filtered, $mean);
        
        return scale($array, $mean, $sd);
    }
    # Ҫȥֵ
    else {
        return scale($array);
    }
}

# һֵΪ0׼Ϊ1
sub scale {
    my $array = shift;
    my $mean = shift;
    my $sd = shift;
    
    $mean = $mean ne "" ? $mean : mean($array);
    $sd = $sd ne "" ? $sd : sd($array, $mean);
    
    return apply($array, sub {($_[0] - $mean)/$sd});
}

# 
sub sample {
    my $array = shift;
    my $size = shift;
    my $replace = shift;
	my $p = shift;
    
    $size = $size || scalar(@$array);
    if(!$replace and scalar(@$array) < $size) {
        croak "ERROR: Size($size) should not be bigger than the sample size with no replacement.\n";
    }
	my $scalar = scalar(@$array);
	if($p eq '') {
		$p = repeat(1/$scalar, $scalar);
	}
	if(scalar(@$array) != scalar(@$p)) {
		croak "ERROR: Length of the vector should be same as the length of the probability.\n";
	}
    my $sample = [];
    
    if($replace) {# Żس
        for(my $i = 0; $i < $size; $i ++) {
            my $ind = _random_int($p);
            push(@$sample, $array->[$ind]);
        }
    }
    else {
		# ȽһarrayĿ
		my $array_copy = [];
		push(@$array_copy, @$array);
        while(scalar(@$array_copy) and $size) {
			my $ind = _random_int($p);
			push(@$sample, $array_copy->[$ind]);
			
			# Ȼѳ鵽ֵɾȥ
			$array_copy = _del_array_item($array_copy, $ind);
			$p = _del_array_item($p, $ind);
			$size --;
		}
    }
    return $sample;
}

# ɾһԪأȻ
sub _del_array_item {
	my $array = shift;
	my $del_i = shift;
	
	return c(subset($array, [0..($del_i-1)]), subset($array, [($del_i+1)..$#$array]));
}

# ոp0#$#p֮
sub _random_int {
	my $p = shift;
	my $sum = sum($p);
	if($sum != 1) {
		$p = apply($p, sub {$_/$sum});
	}
	
	my $rand = rand();
	my $interval_lower = 0;
	my $interval_upper = $p->[0];
	for(my $i = 0; $i < scalar(@$p); $i ++) {
		if($rand >= $interval_lower and $rand < $interval_upper) {
			return $i;
		}
		$interval_lower += $p->[$i];
		if($i + 1 <= scalar(@$p)) {
			$interval_upper += $p->[$i + 1];
		}
		else {
			$interval_upper = 1;
		}
	}
}

# Ψһֵ
sub unique {
    my $array = shift;
    my $hash;
    foreach (@$array) {
        $hash->{$_} = 1;
    }
    return [keys %$hash];
}

# ͨ±01ֵĺȡӼ
# ͨ±ֱȡֵģɵĽܻԭ
# ͨ߼ȡֵģɵĽԭ
sub subset {
    my $array = shift;
    my $index = shift;
    
    if(is_code_ref($index)) {
        my $function = $index;
        return subset($array, which(apply($array, $function)));
    }
    else {
        my $subset = [];
        for(my $i = 0; $i < scalar(@$index); $i ++) {
            push(@$subset, $array->[$index->[$i]]);
        }
        return $subset;
    }
}

# ͨһ߼õΪʱ±
sub which {
    my $logical = shift;
    
    my $which = [];
    for(my $i = 0; $i < scalar(@$logical); $i ++) {
        push(@$which, $i) if($logical->[$i]);
    }
    return $which;
}

# Ľ
sub intersect {
    my $set1 = shift;
    my $set2 = shift;
    
    my $hash1;
    foreach (@$set1) {
        $hash1->{$_} = 1;
    }
    
    my $hash2;
    foreach (@$set2) {
        $hash2->{$_} = 1;
    }
    
    my $intersect;
    foreach (keys %$hash1) {
        push(@$intersect, $_) if($hash2->{$_});
    }
    return $intersect;
}

# Ĳ
sub union {
    my $set1 = shift;
    my $set2 = shift;
    
    my $hash;
    foreach (@$set1) {
        $hash->{$_} = 1;
    }
    foreach (@$set2) {
        $hash->{$_} = 1;
    }
    return [keys %$hash];
}

# collectionжsetеԪ
sub complement {
    my $set = shift;
    my $collection = shift;
    
    my $hash;
    foreach (@$collection) {
        $hash->{$_} = 1;
    }
    my $hash_set;
    foreach (@$set) {
        $hash_set->{$_} = 1;
    }
    
    my $c;
    foreach (keys %$hash) {
        push(@$c, $_) unless($hash_set->{$_});
    }
    return $c;
}

# ֲ̬
sub rnorm {
    my $size = shift;
    my $mean = shift;
    my $sd = shift;
    
    $mean = $mean ne "" ? $mean : 0;
    $sd = $sd ne "" ? $sd : 1;
    
    my $r = [];
    for(my $i = 0; $i < $size; $i ++) {
        my $yita1 = rand(1);
        my $yita2 = rand(1);
        while($yita1 == 0) {
            $yita1 = rand(1);
        }
        while($yita2 == 0) {
            $yita2 = rand(1);
        }
        my $x = sqrt(-2*log($yita1)/log(2.718281828459045))*sin(2*3.1415926*$yita2);
        $x = $mean + $sd*$x;
        push(@$r, $x);
    }
    
    return $r;
}


# algorithm in "Numerical Computation of Multivariate Normal Probabilities - Alan Genz, 1992"
# only for bivariate normal distribution
sub bipnorm {
    my $mean = shift; # mean value
    my $cov = shift;  #covariace matrix
    my $lower_origin = shift;  # lower edge
    my $upper_origin = shift;  #  upper edge
    my $error_cutoff = 0.00000001;      # cutoff for the error
    
    # change each mean as 0
    my $lower;
    my $upper;
    for(my $i = 0; $i < scalar(@$mean); $i ++)
    {
        if(defined($lower_origin->[$i]) and $lower_origin->[$i] ne "") {
            $lower->[$i] = ($lower_origin->[$i] - $mean->[$i])/sqrt($cov->[$i][$i]);
        }
        else {
            $lower->[$i] = "";
        }
        if(defined($upper_origin->[$i]) and $upper_origin->[$i] ne "") {
            $upper->[$i] = ($upper_origin->[$i] - $mean->[$i])/sqrt($cov->[$i][$i]);
        }
        else {
            $upper->[$i] = "";
        }
    }
        
    my $cor;  # correlation 
    
    $cor->[1][0] = $cov->[1][0]/sqrt($cov->[0][0])/sqrt($cov->[1][1]);
    $cor->[0][1] = $cor->[1][0];
    $cor->[0][0] = 1;
    $cor->[1][1] = 1;
    
    my $m = scalar(@$mean);         # dimensions for the distribution, 2
    
    # compute lower triangluat Cholesky factor C for covariace
    # cov = C*C(T)
    my $c;
    $c->[0][0] = sqrt($cor->[0][0]);                  #c[1][1]
    $c->[1][0] = $cor->[1][0]/sqrt($cor->[0][0]); #c[2][1]
    $c->[1][1] = sqrt($cor->[1][1]-$cor->[1][0]**2/$c->[0][0]);
    $c->[0][1] = 0;
    
    my $intsum = 0;
    my $nmax = 1000;
    
    my $d;
    my $e;
    my $f;
    my $delta;
    my $y;
    my $error;
    
    if(defined($lower->[0]) and $lower->[0] ne "") { # is number
        $d->[0] = pnorm($lower->[0]/$c->[0][0]);
    } else {  # -Inf
        $d->[0] = 0;
    }
    if(defined($upper->[0]) and $upper->[0] ne "") {
        $e->[0] = pnorm($upper->[0]/$c->[0][0]);
    } else { # Inf
        $e->[0] = 1;
    }
    $f->[0] = $e->[0] - $d->[0];
    
    sub MonteCarlo {
        my ($c, $d, $e, $lower, $upper) = @_;
        my $w = rand();
        my $foo = $c->[1][0]*qnorm($d->[0]+$w*($e->[0] - $d->[0]));
        my $b, $a;
        if(defined($lower->[1]) and $lower->[1] ne "") { # is number
            $a = pnorm(($lower->[1] - $foo)/$c->[1][1]);
        } else {  # -Inf
            $a = 0;
        }
        if(defined($upper->[1]) and $upper->[1] ne "") {
            $b = pnorm(($upper->[1] - $foo)/$c->[1][1]);
        } else { # Inf
            $b = 1;
        }
        return ($e->[0] - $d->[0]) * ($b - $a);
    }
    
    my $n = 1;
    my ($x, $y, $var,$expected,$error);
    my $foo = MonteCarlo($c, $d, $e, $lower, $upper);
    $x = $foo**2;
    $y = $foo**2;
    while(1) {
        $n ++;
        $foo = MonteCarlo($c, $d, $e, $lower, $upper);
        $x = ($n-1)/$n * $x + $foo**2/$n;
        $y = ($n-1)**2/$n**2 * $y + $foo**2/$n**2 + 2*($n-1)*$foo*sqrt($y)/$n**2;
        $var = $x - $y;
        $expected = sqrt($y);
        $error = sqrt($var)*2.5/sqrt($n);
        if($error < $error_cutoff or $n == $nmax) {
            last;
        }        
    }
    my $p = $expected;
    
    return $p;
}

# ַ
sub paste {
    my $sep = pop;  # һΪָ
    my @args = @_;
    
    return mapply(@args, sub{join $sep, @_});
}

# 
sub seq {
    my $from = shift;
    my $to = shift;
    my $by = shift || 1;
    
    my $seq = [];
    if($from < $to) {
        for(my $i = $from; $i <= $to; $i += $by) {
            push(@$seq, $i);
        }
    }
    else {
        for(my $i = $from; $i >= $to; $i -= $by) {
            push(@$seq, $i);
        }
    }
    return $seq;
}

sub c {
    my @array_refs = @_;
    my $c = [];
    foreach (@array_refs) {
        push(@$c, @$_);
    }
    return $c;
}

sub print_ref {
    my $ref = @_[0];
    my $sep = "\n";
    
    if(is_array_ref($ref)) {
        print "Reference of ARRAY.\n";
        print join $sep, @$ref;
        print "\n";
    } elsif(is_hash_ref($ref)) {
        print "Reference of HASH.\n";
        foreach (keys %$ref) {
            print "$_\t$ref->{$_}$sep";
        }
        print "\n";
    } elsif(is_scalar_ref($ref)) {
        print "Reference of SCALAR.\n";
        print $$ref;
        print "\n";
    } elsif(is_ref_ref($ref)) {
        print "Reference of REF.\n";
        print $$ref;
        print "\n";
    } elsif(is_code_ref($ref)) {
        print "Reference of CODE.\n";
    } else {
        print "@_\n";
    }
    return 1;
}

sub is_numberic {
    my $value = $_[0] + 0;
    if($value =~/^-?\d+\.?\d*$/) {
        return 1;
    } else {
        return 0;
    }
}

sub is_array_ref {
    if($_[0] and Scalar::Util::reftype($_[0])
             and Scalar::Util::reftype($_[0]) eq "ARRAY") {
        return 1;
    }
    else {
        return 0;
    }
}

sub is_hash_ref {
    if($_[0] and Scalar::Util::reftype($_[0])
             and Scalar::Util::reftype($_[0]) eq "HASH") {
        return 1;
    }
    else {
        return 0;
    }
}

sub is_scalar_ref {
    if($_[0] and Scalar::Util::reftype($_[0])
             and Scalar::Util::reftype($_[0]) eq "SCALAR") {
        return 1;
    }
    else {
        return 0;
    }
}

sub is_code_ref {
    if($_[0] and Scalar::Util::reftype($_[0])
             and Scalar::Util::reftype($_[0]) eq "CODE") {
        return 1;
    }
    else {
        return 0;
    }
}

sub is_ref_ref {
    if($_[0] and Scalar::Util::reftype($_[0])
             and Scalar::Util::reftype($_[0]) eq "REF") {
        return 1;
    }
    else {
        return 0;
    }
}

sub max_mutual_str {
  my($str1,$str2) = @_;
  my %cmp;
  my $max;
  my @list1=split '',$str1;
  my @list2=split '',$str2;
  for my $i (0..$#list1){


    for my $j (0..$#list2){


       $cmp{$i.$j}=($list1[$i] eq $list2[$j]) ? (1+$cmp{($i-1).($j-1)}) : 0;
       $max=$cmp{$i.$j} if ($cmp{$i.$j}>$max);
       #print "$cmp{$i.$j}";
    }
    #print "\n";
  }
  for (keys %cmp){
      if ($cmp{$_} == $max ){
         my $offset=substr($_,0,1)+1-$max;
         return substr($str1,$offset,$max);
      }
  }
}

